// Main.cpp : This file contains the 'main' function. Program execution begins and ends there.
//

#include "Yaml.h"
#include <iostream>
#include <chrono>

namespace OpenXcom
{
enum SaveType { SAVE_DEFAULT, SAVE_QUICK, SAVE_AUTO_GEOSCAPE, SAVE_AUTO_BATTLESCAPE, SAVE_IRONMAN, SAVE_IRONMAN_END };
class SavedBattleGame
{
public:
	int _turn;
	void load(const YAML::YamlNodeReader& reader)
	{
		reader.tryRead("turn", _turn);
	}
	void save(YAML::YamlNodeWriter writer) const
	{
		writer.setAsMap();
		writer.write("turn", _turn);
	}
};
class SavedGame
{
public:
	SaveType _type;
	SavedBattleGame* _battle;

	void load(YAML::YamlString yamlString)
	{
		YAML::YamlRootNodeReader reader(yamlString.yaml);
		reader.tryRead("type", _type);
		if (const auto& battleReader = reader["battle"])
		{
			_battle = new SavedBattleGame();
			_battle->load(battleReader);
		}
	}
	YAML::YamlString save() const
	{
		YAML::YamlRootNodeWriter writer;
		writer.setAsMap();
		writer.write("type", _type);
		if (_battle)
			_battle->save(writer["battle"]);
		return writer.emit();
	}
};

} //namespace end

using namespace OpenXcom;

int main()
{ 
	SavedGame game1;
	game1._type = SaveType::SAVE_QUICK;
	game1._battle = new SavedBattleGame();
	game1._battle->_turn = 1;

	//writing test
	YAML::YamlString yamlString = game1.save();

	//reading test
	SavedGame game2;
	game2.load(yamlString);

	assert(game1._type == game2._type);
	assert(game1._battle != game2._battle);
	assert(game1._battle->_turn == game2._battle->_turn);

	//performance test
	using namespace std::chrono;
	std::vector<YAML::YamlRootNodeReader> readers;
	readers.reserve(10);
	auto start = high_resolution_clock::now();
	for (size_t i = 0; i < readers.capacity(); ++i)
	{
		readers.emplace_back("C:\\Dioxine_XPiratez\\user\\mods\\Piratez\\Ruleset\\Piratez.rul", false);
	}
	auto stop = high_resolution_clock::now();
	std::cout << "Parsing time: " << duration_cast<milliseconds>(stop - start).count() / 10 << "ms average" << std::endl;

	start = high_resolution_clock::now();
	for (const auto& reader : readers)
	{
		YAML::YamlString output = reader.emit();
	}
	stop = high_resolution_clock::now();
	std::cout << "Emitting time: " << duration_cast<milliseconds>(stop - start).count() / 10 << "ms average" << std::endl;

	return 0;
}
